##
## jabberif log plugin
##	provides logging into a file and on stdout (in color)
##	this is a core plugin and therefore should not be removed during runtime.
##
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import core_plugin
import time

class log_plugin(core_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'log_plugin'
		self._desc = 'logger'

		self.__log = None
		self.__alltypes = ['error', 'traceback', 'module', 'info', 'connection', 'inmessage', 'outmessage']
		self.__default_config = {	'logfile': '/tmp/jabberif.log',
						'coloroutput':'1',
						'logtofile': "['error']",
						'logtoscreen': "['info', 'error']",
						'colormap': '{}' }
		self._configure()
	
	def _configure(self):
		if self.__log and not self.__log.closed:
			self.__log.close()

		self.__config = self.ph.get_plugin_config(self, self.__default_config)
		self.__tofile = eval(self.__config['logtofile'])
		self.__toscreen = eval(self.__config['logtoscreen'])
		self.__colormap = eval(self.__config['colormap'])
		self.__logfile = self.__config['logfile']

		self.__log = open(self.__logfile, 'a', 0)
	
	def _on_load(self):
		self.ph._register_function(self, 'log', self.log)
		self.log('***** ***** ***** ***** *****', 'info')

	def _on_unload(self):
		self.__log.close()


	####################################################################################################
	## logging
	
	def __file_log(self, message):
		"""log into a file"""
		if not self.__log.closed:
			self.__log.write(time.strftime("[%x %X]") + ' ' + message + '\n')
	
	def __screen_log(self, message, type):
		"""log to the screen"""
		color = ''
		if self.__colormap.has_key(type):
			color = self.__colormap[type]
		print self._colorize(message, color)
		
	def log(self, msg, type='info'):
		## decode unicode
		message = unicode(msg).encode('latin-1', 'replace')
			
		if not type in self.__alltypes:
			self.__screen_log("unknown message type %s" % type, 'error')
			type = 'info'

		if type in self.__tofile:
			self.__file_log(message)
		if type in self.__toscreen:
			self.__screen_log(message, type)

		return message


	####################################################################################################
	## colorizer
	
	def _colorize(self, txt, col=''):
		"""Return ansi-colorized text"""
		if not self.__config['coloroutput'] or col == '': return txt
		cols = { 'red':1, 'green':2, 'yellow':3, 'blue':4, 'purple':5}
		initcode = '\033[;3'
		endcode  = '\033[0m'
		if type(col) == type(1): 
			return initcode + str(col) + 'm' + txt + endcode
		try: return initcode + str(cols[col]) + 'm' + txt + endcode
		except: return txt
		
