##
## jabberif telefon plugin
## 	telephone book access
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import output_plugin
import sys
from string import join,strip
import xmlrpclib
import copy
from optparse import OptionParser
import threading
import re
import traceback


class BeFOptError(Exception):
        def __init__(self, msg):
                self.error = msg

class BeFOptParser(OptionParser):
        def error(self, error):
                raise BeFOptError(error)

class telefon_plugin(output_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'telefon_plugin'
		self._desc = 'private phonebook search'

		self.__default_config = {'url':'http://some:pass@host/path'}
		self._configure()

		self.__server_lock = threading.RLock()
		self.__server = xmlrpclib.ServerProxy(self.__config['url'])

		self.__emptyparams = { "Nachname":"",
	"Vorname":"","Titel":"","Name2":"","Strasse":"","Hausnr":"","Kategorie":"","PLZ":"","Stadt":"","Vorwahl":"","Rufnummer":"","Email":"","URL":"","Nachname_sd":"","Vorname_sd":"", "Phone":"", "Limit":10 }

		self.__parser = BeFOptParser("/tel [options]")	
		self.__parser.remove_option('-h')
		self.__parser.add_option("-n", "--nachname", action="store", type="string", dest="Nachname", default="")
		self.__parser.add_option("-v", "--vorname", action="store", type="string", dest="Vorname", default="")
		self.__parser.add_option("-t", "--titel", action="store", type="string", dest="Titel", default="")
		self.__parser.add_option("-m", "--name2", action="store", type="string", dest="Name2", default="")
		self.__parser.add_option("-s", "--strasse", action="store", type="string", dest="Strasse", default="")
		self.__parser.add_option("-H", "--hausnr", action="store", type="string", dest="Hausnr", default="")
		self.__parser.add_option("-k", "--kategorie", action="store", type="string", dest="Kategorie", default="")
		self.__parser.add_option("-P", "--plz", action="store", type="string", dest="PLZ", default="")
		self.__parser.add_option("-S", "--stadt", action="store", type="string", dest="Stadt", default="")
		self.__parser.add_option("-p", "--vorwahl", action="store", type="string", dest="Vorwahl", default="")
		self.__parser.add_option("-r", "--rufnummer", action="store", type="string", dest="Rufnummer", default="")
		self.__parser.add_option("-e", "--email", action="store", type="string", dest="Email", default="")
		self.__parser.add_option("-u", "--url", action="store", type="string", dest="URL", default="")
		self.__parser.add_option("-N", "--soundex-nachname", action="store", type="string", dest="Nachname_sd", default="")
		self.__parser.add_option("-V", "--soundex-vorname", action="store", type="string", dest="Vorname_sd", default="")
		self.__parser.add_option("-c", "--compl-nr", action="store", type="string", dest="Phone", default="")
		self.__parser.add_option("-l", "--limit", action="store", type="int", dest="Limit", default=10)
		self.__parser.add_option("-h", "--help", action="store_true", dest="Help", default=False)
		self.__parser.add_option("-L", "--verbose", action="store_true", dest="Longoutput", default=False)
	
	def _configure(self):
		self.__config = self.ph.get_plugin_config(self, self.__default_config)
	
	def _on_load(self):
		self.ph._register_command(self, 'telreverse', self.reverse_search, '[nr] reverse tel search', {'level':90})
		self.ph._register_command(self, 'telname', self.name_search, '[(vorname) name] name tel search', {'level':90})
		self.ph._register_command(self, 'tel', self.tel_search, '[options] tel search', {'level':90, 'longhelp':self.__parser.format_help()})
	
	
	####################################################################################################
	## search functions

	def __search(self, search):
		results = False
		self.__server_lock.acquire()
		
		try:
			results = self.__server.tel.search(search)
		except Exception, e:
			self.ph.log("--- stack traceback ---\n%s-----------------------" % join(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), ''), 'traceback')
			
		self.__server_lock.release()
		return results

	def reverse_search(self, args):
		params = args['cmd'][1:]
		params = re.split('[ /-]+', join(params, ' '))
		combined = ""
		vorwahl = ""
		phone = ""
		if len(params) == 1:
			combined = params[0]
		elif len(params) == 2:
			phone = params[1]
			vorwahl = params[0]
		else:
			combined = join(params, '')
			
		search = copy.copy(self.__emptyparams)
		search['Vorwahl'] = vorwahl
		search['Rufnummer'] = phone
		search['Phone'] = combined

		results = self.__search(search)
		if not results:
			args['msg'].send_back('database error. try again later.')
			return
		
		output = self.__format_short(results)
		args['msg'].send_back('\n'+unicode(output, 'iso8859-15'))

	def name_search(self, args):
		params = args['cmd'][1:]
		if len(params) == 1:
			nn = params[0]
			vn = ""
		elif len(params) == 2:
			nn = params[1]
			vn = params[0]
		else:
			args['msg'].send_back('syntax error')
			return
			
		search = copy.copy(self.__emptyparams)
		search['Vorname'] = vn
		search['Nachname'] = nn

		results = self.__search(search)
		if not results:
			args['msg'].send_back('database error. try again later.')
			return

		output = self.__format_short(results)
		args['msg'].send_back('\n'+unicode(output, 'iso8859-15'))

	def tel_search(self, args):
		params = args['cmd'][1:]

		## parse
		try:
			(options, arguments) = self.__parser.parse_args(params)
		except BeFOptError, e:
			args['msg'].send_back(e.error)
			return

		if options.Help:
			args['msg'].send_back(self.__parser.format_help())
			return

		if len(arguments) > 0:
			args['msg'].send_back('error. unknown arguments '+join(arguments, ' ')+'. try /tel -h')
			return

		if len(params) == 0:
			args['msg'].send_back('error. try /tel -h')
			return

		## convert args
		searchparams = copy.copy(self.__emptyparams)
		for key in searchparams.keys():
			searchparams[key] = options.__dict__[key]

		## search 
		results = self.__search(searchparams)
		if results != []:
			args['msg'].send_back('no results')
			return

		## output
		if not results:
			args['msg'].send_back('database error. try again later.')
			return

		## output
		if options.Longoutput:
			output = self.__format_long(results)
		else:
			output = self.__format_short(results)
		
		args['msg'].send_back('\n'+unicode(output, 'iso8859-15'))
	
	def __format_long(self, results):
		out = []
		for row in results:
			rowout = []
			for key in row.keys():
				if row[key] != "":
					rowout.append("%s -->%s<--" % (key, row[key]))
			out.append('\n'+join(rowout, "\n"))
		return join(out, "\n")

	def __format_short(self, results):
		out = []
		for row in results:
			out.append("%s, %s, %s, %s %s, %s-%s" % (row['Nachname'], row['Vorname'], row['Stadt'], row["Strasse"], row["Hausnr"], row['Vorwahl'], row['Rufnummer']))
		return join(out, "\n")
	
