##
## jabberif user plugin
##	user management
##		manage user attributes
##			level: command access level
##
##	this is a core plugin and therefore should not be removed during runtime.
##
##
##	written by Ben Fuhrmannek <bef@erlangen.ccc.de>
##

from baseclasses import core_plugin
from string import join
import re
import marshal
import os

class user_plugin(core_plugin):
	def __init__(self, ph):
		self.ph = ph
		self._name = 'user_plugin'
		self._desc = 'user management'

		self.__users = {}
		
		self.__default_config = {	'user_file': '/tmp/jabberif.users',
						'ops':'' }
		self._configure()
	
	def _configure(self):
		self.__config = self.ph.get_plugin_config(self, self.__default_config)
		self.__userfile = self.__config['user_file']
		self.__ops = re.split('[,\s\n\r]+', self.__config['ops'])
		if self.__userfile:
			if self.__users or not os.path.isfile(self.__userfile):
				self.__save_users()
		self.__load_users()

	
	def _on_load(self):
		self.ph._register_function(self, 'get_userlevel', self.get_userlevel)
		self.ph._register_command(self, 'ulist', self.list_users, 'list user levels', {'level':10 })
		self.ph._register_command(self, 'uload', self.load_users, 'load user levels', {'level':5 })
		self.ph._register_command(self, 'usave', self.save_users, 'save user levels', {'level':5 })
		self.ph._register_command(self, 'uset', self.set_userlevel, '[level, user] set userlevel', {'level':5 })
		self.ph._register_command(self, 'urm', self.remove_user, '[user] remove user', {'level':5 })
		self.ph._register_command(self, 'ulevel', self.show_userlevel, 'show my userlevel', {'level':99 })

	def _on_unload(self):
		self.__save_users()


	####################################################################################################
	## user handling
	
	def __save_users(self):
		fh = file(self.__userfile, "wb")
		marshal.dump(self.__users, fh)
		fh.close()
	
	def __load_users(self):
		self.__users = {}
		
		fh = file(self.__userfile, "rb")
		self.__users = marshal.load(fh)
		fh.close()

		for op in self.__ops:
			self.__set_attrib(op, {'level': 0})

	def __set_attrib(self, user, args):
		if not self.__users.has_key(user):
			self.__users[user] = {}
		for arg in args.keys():
			self.__users[user][arg] = args[arg]
	
	def __get_attrib(self, user, arg):
		if self.__users.has_key(user) and self.__users[user].has_key(arg):
			return self.__users[user][arg]
		return None
		
	def get_userlevel(self, user):
		level = self.__get_attrib(user, 'level')
		if level is not None:
			return level
		return 100
	
	
	####################################################################################################
	## command functions
	
	def list_users(self, args):
		out = []
		for user in self.__users.keys():
			out.append("%3d %s" % (self.get_userlevel(user), user))
		args['msg'].send_back('\n'+join(out, '\n'))
		
	def load_users(self, args):
		self.__load_users()
		args['msg'].send_back('loaded users db')
		
	def save_users(self, args):
		self.__save_users()
		args['msg'].send_back('saved users db')

	def set_userlevel(self, args):
		if len(args['cmd'][1:]) < 2:
			args['msg'].send_back('syntax error')
			return
		user = args['cmd'][2]
		try:
			level = int(args['cmd'][1])
		except ValueError:
			args['msg'].send_back('syntax error')
			return
		
		self.__set_attrib(user, {'level':level})
		args['msg'].send_back("set level %d for %s" % (level, user))
	
	def remove_user(self, args):
		if len(args['cmd'][1:]) < 1:
			args['msg'].send_back('syntax error')
			return
		user = args['cmd'][1]
		if self.__users.has_key(user):
			self.__users.pop(user)
			args['msg'].send_back('user successfully removed')
		else:
			args['msg'].send_back('there is no such user')
	
	def show_userlevel(self, args):
		args['msg'].send_back("your userlevel is %d" % self.ph.get_userlevel(args['msg'].get_uid()))

